package parMSpindle;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.image.codec.jpeg.*;
import java.text.*;

/* Modified from.....
 * Class CapturingCanvas3D, using the instructions from the Java3D 
    FAQ pages on how to capture a still image in jpeg format.

    A capture button would call a method that looks like


    public static void captureImage(CapturingCanvas3D MyCanvas3D) {
	MyCanvas3D.writeJPEG_ = true;
	MyCanvas3D.repaint();
    }


    Peter Z. Kunszt
    Johns Hopkins University
    Dept of Physics and Astronomy
    Baltimore MD
*/

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
Copyright (C) <2008>  <Jonathan B. Alberts>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

public class CapturingCanvas3D extends Canvas3D  {
	public boolean rendering = false;
    public boolean writeJPEG_ = false;
    public boolean useAltCaptureName = false;
    public String captureName;
    public String capturePath;
    public String altCaptureName;
    public String fileName;
    private int postSwapCount_;
    DecimalFormat countFormat = new DecimalFormat ("#0000.#;#0000.#");
    J3DGraphics2D g2D;
    
    public CapturingCanvas3D(GraphicsConfiguration gc) {
    		super(gc);
    		postSwapCount_ = 0;
    }
    
    public void preRender() {
    	
    }
    
    public void do2DDrawing() {
		rendering = true;
		g2D.setColor(Env.canvasTextColor);
		String showString = "";
		String sepString = "   ";
		if (Env.showTime.isActive()) { showString += ParMSpindle_Graphics.getTimeString() + sepString; }
		if (Env.cantileverElasticityTst | Env.simpleSupportElasticityTst) {
			g2D.setFont(Env.elasticityFont);
			showString += ParMSpindle_Graphics.getTstFilLengthString() + sepString;
			if (Env.filTorqSpring.isActive()) {
				showString += ParMSpindle_Graphics.getCurrentElasticity() + sepString;
			}
			showString += ParMSpindle_Graphics.getCurrentDeflectionString() + sepString;
			showString += ParMSpindle_Graphics.getExpectedDeflectionString() + sepString;
			showString += ParMSpindle_Graphics.getRelaxationDispString() + sepString;
		} else if (Env.testingPersistenceLength) {
			showString += ParMSpindle_Graphics.getPersistenceTstLengthString() + sepString;
			//showString += ParMSpindle_Graphics.getLastSegCorrelationString() + sepString;
		} else {
			if (Env.showConc.isActive()) { showString += ParMSpindle_Graphics.getConcString() + sepString; }
			if (Env.showNonHydroConc.isActive()) { showString += ParMSpindle_Graphics.getNonHydroConcString() + sepString; }
			if (Env.showFilCt.isActive()) { showString += ParMSpindle_Graphics.getFilCtString() + sepString; }
			if (Env.showFilSegCt.isActive()) { showString += ParMSpindle_Graphics.getFilSegCtString() + sepString; }
			if (Env.showFilLinkCt.isActive()) { showString += ParMSpindle_Graphics.getFilLinkCtString() + sepString; }
			if (Env.showMonCt.isActive()) { showString += ParMSpindle_Graphics.getMonCtString() + sepString; }
		}
		g2D.drawString(showString,10,20);
		if (Env.cantileverElasticityTst) { g2D.drawString(Env.elasReportString, 10, 180); }
		BufferedImage filLengthHistoImage = FilHistoControl.hPanel.getCurrentImage();
		if ((filLengthHistoImage != null) & (Env.makeFilLengthHisto.isActive())) {
			g2D.drawImage(filLengthHistoImage,Env.histX.getIntValue(),Env.histY.getIntValue(),null);
		}
		g2D.flush(false);
		
    }
    
    public void postRender () {
    	if ((Env.infoIn2D) & (Env.paintOn)) {
    		g2D = this.getGraphics2D();
    		do2DDrawing(); 
    	}
    	rendering = true;
    }

    public void postSwap() {
	    	if (writeJPEG_) {
	    		rendering = true;
		    int width = this.getWidth();
		    int height = this.getHeight();
		    GraphicsContext3D  ctx = getGraphicsContext3D();
		    // The raster components need all be set!
		    Raster ras = new Raster(
	                   new Point3f(-1.0f,-1.0f,-1.0f),
			   Raster.RASTER_COLOR,
			   0,0,
			   width,height,
			   new ImageComponent2D(
	                             ImageComponent.FORMAT_RGB,
				     new BufferedImage(width,height,
						       BufferedImage.TYPE_INT_RGB)),
			   null);
	
		    ctx.readRaster(ras);
	
		    // Now strip out the image info
		    BufferedImage img = ras.getImage().getImage();
	
		    // write that to disk....
		    try {
		    	if (useAltCaptureName) {
		    		fileName = capturePath +"/" + altCaptureName + ".jpg";
		    	} else {
		    		fileName = capturePath +"/" + FileOps.makeNameFromIntStep(captureName,Env.counter) + ".jpg";
		    	}
		    	System.out.print ("Writing " + fileName + " ... ");
		    	FileOutputStream out = new FileOutputStream(fileName);
			JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
			JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(img);
			if ((Env.jpegQuality.getFloatValue() > 1.0) | (Env.jpegQuality.getFloatValue() < 0)) {
				System.out.println ("JPEG quality out of bounds.. fix that will you.");
				param.setQuality(1.0f,false); // quality of the JPEG
			} else {
				param.setQuality(Env.jpegQuality.getFloatValue(),false); // quality of the JPEG
			}
			encoder.setJPEGEncodeParam(param);
			encoder.encode(img);
			writeJPEG_ = false;
			useAltCaptureName = false;
			out.close();
		    } catch ( IOException e ) {
			System.out.println("I/O exception!");
		    }
		    postSwapCount_++;
			System.out.println("done.");
		}
	    	rendering = false;
    }
    
    public void setAltCapture (String altName) {
    		altCaptureName = altName;
    		useAltCaptureName = true;
    }
    
    public boolean isWritingJPEG() {
    		return writeJPEG_;
    }
    
    public boolean isRendering () {
    		return rendering;
    }
    
    public boolean firstRender () {
    	if (postSwapCount_ == 0) { return true; } else { return false; }
    }
    
    public void resetFileCt () {
    	postSwapCount_ = 0;
    }
    
}
