package parMSpindle;
/* Monomer... */

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import com.sun.j3d.utils.geometry.Sphere;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Monomer {
	// monomer array.. used only in drawing from QK files
	// the following are only used in rendering... not in the sim itself
	static final int maxMons = (int)4e6;
	static final Monomer [] theMonomers = new Monomer[maxMons];
	static int monCt = 0;
	static int monRenderCt = 0; // counter for how many mons rendered in a QK file read
	boolean graphicsIn = false;	// flag for graphics currently in scene or not
	boolean hydrolyzable = true;		// flag to indicate if this is a hydrolyzable ParM monomer
	boolean plasmidBound = false;		// flag used to change appearance if plasmid bound

	static final int MINUSEND = 0;
	static final int PLUSEND = 1;
	static final int MINUSSEED = 4;
	
	static final int ATPstate = 10;
	static final int ADPPistate = 11;
	static final int ADPstate = 12;
	
	int endType;
	int nucleotideState = ATPstate;
	boolean removeMe = false;
	
	// for 3D graphics
	static boolean appearancesInitialized = false;
	static float shiny = 128.0f;
	/* gray colors
	static final Color3f atpColor = new Color3f(0.0f,0.0f,1.0f);
	static final Color3f adppiColor = new Color3f(1.0f,0.5f,0.0f);
	static final Color3f adpColor = new Color3f(1.0f,0.0f,0.0f);
	 */
	static final Color3f emmissiveC = new Color3f(0f,0f,0f);
	static final Color3f specularC = new Color3f(1f,1f,1f);
	static final Color3f ambientC = new Color3f(0f,0f,0f);
	static final Color3f atpColor = new Color3f(0.0f,0.0f,1.0f);
	static final Color3f adppiColor = new Color3f(1.0f,0.5f,0.0f);
	static final Color3f adpColor = new Color3f(1.0f,0.0f,0.0f);
	static final Color3f plasADPColor = new Color3f(adpColor);//new Color3f(0.8f,0.8f,0.8f);
	static final ColoringAttributes atpCA = new ColoringAttributes(atpColor, ColoringAttributes.FASTEST);
	static final ColoringAttributes adppiCA = new ColoringAttributes(adppiColor, ColoringAttributes.FASTEST);
	static final ColoringAttributes adpCA = new ColoringAttributes(adpColor, ColoringAttributes.FASTEST);
	static final ColoringAttributes plasADPCA = new ColoringAttributes(plasADPColor, ColoringAttributes.FASTEST);
	static final Material atpMat = new Material(ambientC,atpColor,atpColor,specularC,shiny);
	static final Material adppiMat = new Material(ambientC,adppiColor,adppiColor,specularC,shiny);
	static final Material adpMat = new Material(ambientC,adpColor,adpColor,specularC,shiny);
	static final Material plasADPMat = new Material(ambientC,plasADPColor,plasADPColor,specularC,shiny);
	static Appearance atpApp = new Appearance();
	static Appearance adppiApp = new Appearance();
	static Appearance adpApp = new Appearance();
	static Appearance plasADPApp = new Appearance();
	static final LineAttributes monLineAttributes = new LineAttributes(2.0f,LineAttributes.PATTERN_SOLID,true);
	boolean graphicsInitialized = false;
	LineArray theMonLine;// = new LineArray(2,LineArray.COORDINATES);
	Shape3D monShape; //= new Shape3D();
	BranchGroup monHelixG;// = new BranchGroup();
	BranchGroup monSphereG;// = new BranchGroup();
	Sphere monSphere;
	TransformGroup monTG;// = new TransformGroup();
	Transform3D monT3D;// = new Transform3D();
	Vector3d monVec3D = new Vector3d(); // need to instantiate this one, updated for QK file writes
	
	// testing
	static double kHydroRate;
	static double kDissocRate;
	
	Monomer frontMon = null;
	Monomer backMon = null;
	
	static Monomer plusGhost = new Monomer ();
	static Monomer minusGhost = new Monomer ();
	
	public Monomer(){
		if (!Env.remote) { initializeGraphics(); }
	}
	
	public Monomer (Monomer myNeighbor, int endType){
	   switch (endType) {
	     case MINUSEND :
		    	frontMon = myNeighbor;
		    	myNeighbor.backMon = this;
		    	this.backMon=minusGhost;
		    	break;
	     case PLUSEND :
		    	backMon = myNeighbor;
		    	myNeighbor.frontMon = this;
		    	this.frontMon=plusGhost;
		    	break;
	     case MINUSSEED:
	    		this.backMon=minusGhost;
	    		break;
	     }
	   setStateATP ();
	   if (!Env.remote) { initializeGraphics(); }
	   
	}
	
	public Monomer (Monomer myNeighbor, int endType, int state){
		   switch (endType) {
		     case MINUSEND :
		    	 	frontMon = myNeighbor;
		    	 	myNeighbor.backMon = this;
		    	 	this.backMon=minusGhost;
		    	 	break;
		     case PLUSEND :
		    	 	backMon = myNeighbor;
		    	 	myNeighbor.frontMon = this;
		    	 	this.frontMon=plusGhost;
		    	 	break;
		     case MINUSSEED:
		    		this.backMon=minusGhost;
		    		break;
		   }
		   switch (state) {
		   case ATPstate :
			  setStateATP ();
			  //addATPMon(this);
			  break;
		   case ADPPistate :
			  setStateADPPi ();
			  //addADPPiMon(this);
			  break;
		   case ADPstate :
			  setStateADP ();
			  break;
		}   
		if (!Env.remote) { initializeGraphics(); }
	}
	
	
	public static void polymerize (Monomer myNeighbor, FilSegment myFilament, int endType, boolean canHydrolyze){
		Monomer brandnewMon = new Monomer(myNeighbor, endType);
		switch (endType) {
		case Monomer.MINUSEND:
			myFilament.minusMon = brandnewMon;
			break;
		case Monomer.PLUSEND:
			myFilament.plusMon = brandnewMon;
			break;
		case Monomer.MINUSSEED:
			myFilament.minusMon = brandnewMon;
			break;
		}
		if (!Env.remote) { brandnewMon.addGraphics(myFilament); }
		brandnewMon.hydrolyzable = canHydrolyze;
		//if (canHydrolyze) { System.out.println ("made a normal monomer in polymerize"); }
	}
	
	
	public void  depolymerize (FilSegment myFilament, int endType){
		switch (endType) {
	     case MINUSEND :
	    	 	if (frontMon != plusGhost) { 
	    	 		frontMon.backMon=minusGhost; 
	    	 		myFilament.minusMon = frontMon;
	    	 	}
	    	 	break;
	     case PLUSEND :
	    	 	if (backMon != minusGhost) { 
	    	 		backMon.frontMon=plusGhost; 
	    	 		myFilament.plusMon = backMon;
	    	 	}
	    	 	break;
	     }
		frontMon = null;
		backMon = null;
		nullify();
	}
	
	public void setState (int state) {
		switch(state) {
		case Monomer.ATPstate:
			setStateATP();
			return;
		case Monomer.ADPPistate:
			setStateADPPi();
			return;
		case Monomer.ADPstate:
			setStateADP();
			return;
		}
		setStateATP();	// default if we somehow fall through cases
	}
	
	public void setStateATP () {
		nucleotideState = ATPstate;
	}
	
	public void setStateADPPi () {
		nucleotideState = ADPPistate;
	}
	
	public void setStateADP () {
		nucleotideState = ADPstate;
	}
		
	
	public void checkHydrolysis (){
		switch (nucleotideState) {
		case Monomer.ADPstate:
			break;
		case Monomer.ATPstate:
			hydrolize();
			break;
		case Monomer.ADPPistate:
			dissociate();
			break;
		}
	}
	
	
	public void hydrolize (){
		if (!hydrolyzable) { return; }
		
		if (Env.kTerminalHydro.isActive() & isTerminalMonomer()) {
			if (Env.mtRNG.raw() < Env.kTerminalHydro.getValue()*Env.biochemDeltaT.getValue()) {
				setStateADPPi ();
	        }
		} else {
			double adjFactor = 1;
			if (Env.kCoopHydro.isActive()) { 
				try {
					if (frontMon.isADP()) { adjFactor = Env.kCoopHydro.getValue(); }
					if (backMon.isADP()) { adjFactor = Env.kCoopHydro.getValue(); }
				} catch (NullPointerException npe) { System.out.println ("null pointer exception in Monomer.hydrolyze()"); }
			}
			
			if (Env.mtRNG.raw() < adjFactor*Env.kHydrolysis.getValue()*Env.biochemDeltaT.getValue()) {
				setStateADPPi ();
	        }
		}
	}
	

	public void dissociate() {
		if (Env.mtRNG.raw() < Env.kDissociation.getValue()*Env.biochemDeltaT.getValue()) {
		     setStateADP ();
		}
	}	  
	
	public static void dissociationSimForAll () {
		for (int i=0;i<monCt;i++) {
			theMonomers[i].dissociate();
		}
	}

	public int getState () {
		return nucleotideState;
	}
	
	public void getPosition (Vector3d vec3d) {
		monT3D.get(vec3d);
	}
	
	public boolean isATP() {
		if (nucleotideState == ATPstate) { return true; } else { return false; }
	}
	public boolean isADPPi () {
		if (nucleotideState == ADPPistate) { return true;} else { return false; }
	}
	public boolean isADP () {
		if (nucleotideState == ADPstate){ return true; } else { return false; }
	}
	
	public boolean notATP() {
		if (nucleotideState != ATPstate) { return true; } else { return false; }
	}
	public boolean notADPPi () {
		if (nucleotideState != ADPPistate) { return true;} else { return false; }
	}
	public boolean notADP () {
		if (nucleotideState != ADPstate){ return true; } else { return false; }
	}
	
	public boolean isTerminalMonomer() {
		// a terminal monomer is either of the last two monomers on an end
		try {
			if (frontMon.equals(plusGhost)) { return true; }
			if (backMon.equals(minusGhost)) { return true; }
			
			if (frontMon.frontMon.equals(plusGhost)) { return true; }
			if (backMon.backMon.equals(minusGhost)) { return true; }
		} catch (NullPointerException npe) { return false; }
		
		return false;
	}
		
	public void chooseBiochemAppearance() {
		try {
		switch (nucleotideState) {
			case ATPstate:
				monShape.setAppearance(atpApp);
				monSphere.getShape().setAppearance(atpApp);
				break;
			case ADPPistate:
				monShape.setAppearance(adppiApp);
				monSphere.getShape().setAppearance(adppiApp);
				break;
			case ADPstate:
				monShape.setAppearance(adpApp);
				if (plasmidBound) {
					monSphere.getShape().setAppearance(plasADPApp);
				} else {
					monSphere.getShape().setAppearance(adpApp);
				}
				break;
			}
		} catch (Exception e) { System.out.println ("Exception in chooseBiochemAppearance()"); }
	}
	
	public static void initializeAppearances () {
		atpApp.setColoringAttributes(atpCA);
		atpApp.setMaterial(atpMat);
		atpApp.setLineAttributes(monLineAttributes);
		adppiApp.setColoringAttributes(adppiCA);
		adppiApp.setMaterial(adppiMat);
		adppiApp.setLineAttributes(monLineAttributes);
		adpApp.setColoringAttributes(adpCA);
		adpApp.setMaterial(adpMat);
		adpApp.setLineAttributes(monLineAttributes);
		plasADPApp.setColoringAttributes(plasADPCA);
		plasADPApp.setMaterial(plasADPMat);
		plasADPApp.setLineAttributes(monLineAttributes);
		
		appearancesInitialized = true;
	}
	
	public void initializeGraphics () {
		// initialize Java3D objects needed
		theMonLine = new LineArray(2,LineArray.COORDINATES);
		monShape = new Shape3D();
		monHelixG = new BranchGroup();
		monSphereG = new BranchGroup();
		monTG = new TransformGroup();
		monT3D = new Transform3D();
		
		if (!appearancesInitialized) { initializeAppearances(); }
		monHelixG.setCapability(BranchGroup.ALLOW_DETACH);
		monSphereG.setCapability(BranchGroup.ALLOW_DETACH);
		theMonLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		monShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		monShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		monTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		monSphere = new Sphere((float)(Env.parMHalfMonoSize),Sphere.GENERATE_NORMALS, Env.monomerTessalation, atpApp);
		monSphere.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		
		monShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		monShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		theMonLine.setCoordinate(0,Env.farfarAway);
		theMonLine.setCoordinate(1,Env.farfarAway);
		monShape.setGeometry(theMonLine);
		monHelixG.addChild(monShape);
	
		monTG.addChild(monSphere);
		monTG.setTransform(monT3D);
		monSphereG.addChild(monTG);
			
		chooseBiochemAppearance();
		
		graphicsInitialized = true;
	}
	
	public void resetGraphics (FilSegment fil) {
		if (Env.helixSpheres) {
			monHelixG.detach();
			fil.G.addChild(monSphereG);
		} else {
			monSphereG.detach();
			fil.G.addChild(monHelixG);
		}
	}
	
	public void reassignGraphics (FilSegment oldFil, FilSegment newFil) {
		if (Env.helixSpheres) {
			oldFil.G.removeChild(monSphereG);
			newFil.G.addChild(monSphereG);
		} else {
			oldFil.G.removeChild(monHelixG);
			newFil.G.addChild(monHelixG);
		}
	}
	
	public void updateGraphics (Pt3D start, Pt3D stop) {
		if (!Env.helixSpheres) {
			theMonLine.setCoordinate(0,start);
			theMonLine.setCoordinate(1,stop);
			chooseBiochemAppearance();
		} else {
			chooseBiochemAppearance();
			start.copyToVector3d(monVec3D);
			monT3D.setTranslation(monVec3D);
			monTG.setTransform(monT3D);
		}
	}
	
	public void setFromQKInfo (Pt3D loc, int state, boolean plasBound) {
		loc.copyToVector3d(monVec3D);
		setState(state);
		plasmidBound = plasBound;
		chooseBiochemAppearance();
		monT3D.setTranslation(monVec3D);
		monTG.setTransform(monT3D);
	}
	
	public void updatePosition (Pt3D loc) {
		loc.copyToVector3d(monVec3D);
	}
	
	
	public void addGraphics (FilSegment fil) {
		if (Env.helixSpheres) {
			fil.G.addChild(monSphereG);
		} else {
			fil.G.addChild(monHelixG);
		}
	}
	
	public void addGraphics (BranchGroup baseG) {
		if (Env.helixSpheres) {
			baseG.addChild(monSphereG);
		} else {
			baseG.addChild(monHelixG);
		}
		graphicsIn = true;
	}
	
	public void detachGraphics () {
		monSphereG.detach();
		graphicsIn = false;
	}

	public void nullify () {
		removeMe = true;
		if (!Env.remote) {
			monHelixG.detach();
			monSphereG.detach();
		}
		monHelixG = null;
		monSphereG = null;
		theMonLine = null;
		monShape = null;
		monSphere = null;
		monTG = null;
		monT3D = null;
		monVec3D = null;
	}
	
	public static void removeAll() {
		for (int i=0;i<monCt;i++) {
			theMonomers[i].detachGraphics();
			theMonomers[i].nullify();
		}
	}
	
	
}

