package parMSpindle;
/*
 * Plasmid.java
 *
 */

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;

import java.lang.Math.*;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.File;
import com.sun.j3d.utils.image.TextureLoader;
import java.net.URL;

public class Plasmid extends Thing {
	static Plasmid [] thePlasmids = new Plasmid [500];	// array holding all plasmids
	static int plasmidCt = 0;			//
	private int boundFilaments = 0;			// counter for number of bound ParM filaments
	private int registeredFilCt = 0;
	private int [] registeredFilIds = new int [50];
	static int maxParRparCs = 100;
	Pt3D [] parRparCs = new Pt3D [maxParRparCs];
	int parRparCCt = 0;
	ValueTracker bugCollisionMag = new ValueTracker(Env.bugPlasmidCollisionsToSum);
	ValueTracker bugCollidingTrack = new ValueTracker(Env.bugPlasmidCollisionsToSum);
	Color plasmidColor = Color.green;	// color for drawing
	Pt3D aVec = new Pt3D(); //recycled

	Sphere mySphere;
	Sphere lightSphere;
	Box lightBox;
	BranchGroup centerGroup = new BranchGroup();
	Sphere [] parRparCSpheres = new Sphere [maxParRparCs];
	static boolean cubeAppsMade = false;
	static String [] cubePics = new String [] {"albertsFramed.jpg","trichetFramed.jpg","garnerFramed.jpg","mullinsFramed.jpg","CCDlogo.jpg","UCSFlogo.jpg"};
	static Appearance[] cubeApps = new Appearance [6];
	Vector3d coordVec3d = new Vector3d();
	static Color3f ambientC = new Color3f(0.0f,0.0f,0.0f);
	//static Color3f diffuseC = new Color3f(0.0f,0.3f,0.6f);
	static Color3f diffuseC = new Color3f(0.8f,0.8f,0.8f);
	static Color3f specularC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f emissiveC = new Color3f(0.0f,0.0f,0.0f);
	
	static Color3f ambientIC = new Color3f(0.0f,0.0f,0.0f);
	static Color3f diffuseIC = new Color3f(0.8f,0.8f,0.8f);
	static Color3f specularIC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f emissiveIC = new Color3f(0.5f,0.8f,1.0f);
	static float shiny = 60.0f;
	static int pLightCt = 2;
	PointLight [] pLights = new PointLight[pLightCt];
	
	public Plasmid (Pt3D initCoord) {
		super(initCoord);
		addPlasmid(this);
		calculateProperties();
		initialize();
	}
	
	public static double defaultTransDiff () {
		double defaultTransGam = 6*Math.PI*Env.aeta.getValue()*(1.0e-6*Env.plasmidRadius.getValue());
		return Env.Boltz*Env.tempK/defaultTransGam;
	}
	
	public static double defaultRotDiff () {
		double defaultRotGam = 8*Math.PI*Env.aeta.getValue()*Math.pow(1.0e-6*Env.plasmidRadius.getValue(),3);
		return Env.Boltz*Env.tempK/defaultRotGam;
	}
	
	public void calculateProperties () {
		if (Env.plasmidTransDiff.isActive()) {
			bTransDiff.x = Env.plasmidTransDiff.getValue();
			bTransDiff.y = bTransDiff.x;
			bTransDiff.z = bTransDiff.x;
			bTransGam.div(Env.Boltz*Env.tempK,bTransDiff);		// Einstein's relation gamma=kT/D
		} else {
			double radiusM = getRadius()*1.0e-6;
			bTransGam.x = 6*Math.PI*Env.aeta.getValue()*radiusM;
			bTransGam.y = bTransGam.x;
			bTransGam.z = bTransGam.x;
			bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		}
		
		if (Env.plasmidRotDiff.isActive()) {
			bRotDiff.x = Env.plasmidRotDiff.getValue();
			bRotDiff.y = bRotDiff.x;
			bRotDiff.z = bRotDiff.x;
			bRotGam.div(Env.Boltz*Env.tempK,bRotDiff);		// Einstein's relation gamma=kT/D
		} else {
			double radiusM = getRadius()*1.0e-6;
			bRotGam.x = 8*Math.PI*Env.aeta.getValue()*Math.pow(radiusM,3);	// drag for turning about x
			bRotGam.y = bRotGam.x;
			bRotGam.z = bRotGam.x;
			bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);		// Einstein's relation gamma=kT/D
		}
	}
	
	public void initialize () {
		// this method assumes the unit x and y vectors have been set (though maybe not orthogonal), or are unchanged
		// determine z-unit vectors, then reset y-unit vector to ensure orthogonality with uVec
		zVec.cross(uVec, yVec);
		yVec.cross(zVec, uVec);
		// find the transformation matrices at this time step
		transMat ();
	}
	
	public void incForceSum (Pt3D F, Pt3D vec) {
		incForceSum(F);
		//aVec.unitVec(vec);
		//double FdotVecVal = Pt3D.Dot(F, aVec);
		//aVec.scale(FdotVecVal);
	}
	
	public void moveThing () {
		// Given the forces/torques at this time point... move with explicit Euler approximation to ODE solution
		
		// calculate properties...they don't typically change during a run unless gui-usered
		calculateProperties();
		
		// first check that forceSum and torqueSum aren't wacky... exit method if they are
		if (!forceSum.checkPt3D()) { return; }
		if (!torqueSum.checkPt3D()) { return; }
		
		// Work in coordinates aligned with the plasmid... transform forces and torques into body-fixed axis....
		bForceSum.XTox(this, forceSum);
		bTorqueSum.XTox(this, torqueSum);
		if (!Env.plasmidBrownianMotionOff) {
			// add random forces, given in body-fixed frame
			calcRandomForces();
			// then add to force and torque sums
			bForceSum.inc(randForces);
			bTorqueSum.inc(randTorques);
		}
		
		if (false) {
			Pt3D constantF = new Pt3D(xUnitVector);
			constantF.scale(1e-13);
			constantF.XTox(this);
			bForceSum.inc(constantF);
		}
		// now that the forces and torques are in the body fixed frame, we apply the eoms....
		bVeloc.div(1.0e6, bForceSum, bTransGam);		// in micron/sec
		bAngVeloc.div(bTorqueSum, bRotGam);			// in radians/sec
		
		// ** before progressing .... check that bVeloc and bAngVeloc are not NaN... exit if wacky
		if (!bVeloc.checkPt3D()) { talkln ("** problem with bVeloc for filament " + String.valueOf(this)); return; }
		if (!bAngVeloc.checkPt3D()) { talkln ("** problem with bAngVeloc for filament " + String.valueOf(this)); return; }
		
		// ....then transform back into fixed coordinate frame
		veloc.xToX(this, bVeloc);
		coord.inc(Env.deltaT.getValue(), veloc);
		
		// to apply the body-fixed angular velocities, approximate new unit vector from arc of rotations.. good for small rotations
		// for uVec
		double uVecTransInZ = -bAngVeloc.y * Env.deltaT.getValue();	// arclength out at 1 micron
		double uVecTransInY = bAngVeloc.z * Env.deltaT.getValue();
		uVec.setVals(1,uVecTransInY,uVecTransInZ);	// in body-fixed, not a unit vector yet
		uVec.xToX(this);	// make in fixed-frame, not a unit vector yet
		uVec.unitVec();		// make a unit vector
		
		// for yVec 
		double yVecTransInX = - uVecTransInY;
		double yVecTransInZ = bAngVeloc.x * Env.deltaT.getValue();	// arclength at 1 micron
		yVec.setVals(yVecTransInX, 1, yVecTransInZ);
		yVec.xToX(this);
		yVec.unitVec();
		
		initialize();
	}
	
	public void step () {
		//increment counters that control how often different sims occur
		collCheckCt++;
		if (!Env.biochemOnly.isActive()) {
			if (!Env.bugOff & (collCheckCt >= collisionCheckInt | Env.simulationTime == 0)) {
				checkBugCollision();		// these should add forces and torques to forceSum and torqueSum
				collCheckCt = 0;
			}
		
			moveThing();
		}
		
		initialize();
	
		// reset counters etc.
		resetCounters();
	}
	
	public void checkBugCollision () {
		CollisionEvent cE = theBug.amIcolliding(coord,getRadius());
		if (cE == null) { 
			bugCollidingTrack.registerValue(0);
			if (bugCollidingTrack.averageVal() > 0) { collidedWithBugThisStep = true;} else { collidedWithBugThisStep = false; }
			return; 
		} 
		// collision!
		double mag = Env.plasmidCollisionAttn.getValue()*1.0e-6*cE.delta*bTransGam.x/Env.collisionDeltaT.getValue();
		bugCollisionMag.registerValue(mag);
		forceSum.inc(Pt3D.Scale(bugCollisionMag.sum(),cE.forceUVec));
		bugCollidingTrack.registerValue(1);
		if (bugCollidingTrack.averageVal() > 0) { collidedWithBugThisStep = true;} else { collidedWithBugThisStep = false; }
	}
	
	public static void checkPlasmidCollision () {
		//for (int i=0;i<plasmidCt;i++) { thePlasmids[i].plasmidColor = Color.pink; }
		for (int i=0; i<plasmidCt; i++) {
			Plasmid iP = thePlasmids[i];
			for (int p=i+1;p<plasmidCt;p++) {
				Plasmid pP = thePlasmids[p];
				double pDist = Pt3D.ptDist(iP.coord, pP.coord);
				if (pDist<pP.getRadius()+iP.getRadius()) {
					double impingedist = pP.getRadius()+iP.getRadius() - pDist;
				    Pt3D iVec = Pt3D.UnitVec(pDist, iP.coord, pP.coord);
					Pt3D pVec = Pt3D.Reverse(iVec);
					double mag = (Env.plasmidCollisionAttn.getValue()*1.0e-6*impingedist/Env.collisionDeltaT.getValue())/(1/iP.bTransGam.x+1/pP.bTransGam.x);
					iP.forceSum.inc(Pt3D.Scale(mag,iVec));
					pP.forceSum.inc(Pt3D.Scale(mag,pVec));
					//iP.plasmidColor = Color.green;
					//pP.plasmidColor = Color.green;
				}
			}
		}
	}
	
	public void resetCounters() {
		super.resetCounters();	// call the generic Thing method
		registeredFilCt = 0;
	}
	
	public double getRadius () {
		return Env.plasmidRadius.getValue();
	}
	
	public boolean canBindFilament () {
		if (boundFilaments < Env.filsPerPlasmid.getValue()) { return true; }
		return false;
	}
	
	public void filamentOn () {
		boundFilaments ++;
	}
	
	public void filamentOff () {
		boundFilaments --;
		if (boundFilaments < 0) { boundFilaments = 0; }
	}
	
	public void registerFilament (FilSegment boundSeg) {
		registeredFilIds[registeredFilCt] = boundSeg.filID;
		registeredFilCt++;
	}
	
	public boolean filamentBound (int filID) {
		for (int i=0;i<registeredFilCt;i++) {
			if (filID == registeredFilIds[i]) { return true; }
		}
		return false;
	}
	
	public static boolean filamentBoundToAnyPlasmid (int filID) {
		for (int i=0;i<plasmidCt;i++) {
			if (thePlasmids[i].filamentBound(filID)) { return true; }
		}
		return false;
	}
	
	public void drawYourself (Graphics g, double scale, double [] offset) {
		g.setColor(plasmidColor);
		
		int xCenter = (int) ((coord.x-offset[0])*scale);
		int yCenter = (int) ((coord.y-offset[1])*scale);
		int pixelRadius = (int) (getRadius()*scale);
		int pixelDiameter = (int) (2*getRadius()*scale);
		
		g.drawOval(xCenter-pixelRadius,yCenter-pixelRadius,pixelDiameter,pixelDiameter);
	}
	
	public static void plasmidsLightSwitch (boolean on) {
		for (int i=0;i<Plasmid.plasmidCt;i++) {
			for (int j=0;j<pLightCt;j++){
				thePlasmids[i].pLights[j].setEnable(on);
			}
		}
	}
	
	public static File cwd() {
		return new File(System.getProperty("user.dir","."));
	}
	 
	public static File resDir() {
		String p = System.getProperty("jexepack.resdir");
		return (p!=null) ? new File(p) : cwd();
	}
	
	public static void makeCubeApps () {
		infoCCD.ResourceGetter rG = new infoCCD.ResourceGetter(infoCCD.Info.runType);
		TextureLoader cubePicLoader;
		for (int i=0;i<6;i++) {
			URL wherePicsAt = rG.getResource("cubePics/"+cubePics[i]);
			cubePicLoader = new TextureLoader(wherePicsAt,ParMSpindle.parMGraphics);
			cubeApps[i] = new Appearance();
			cubeApps[i].setTexture(cubePicLoader.getTexture());
		}
		cubeAppsMade = true;
	}
	
	public static void switchLightCenters () {
		for (int i=0;i<plasmidCt;i++) {
			thePlasmids[i].centerGroup.detach();
			thePlasmids[i].centerGroup.removeAllChildren();
			if (Env.plasmidFaces) {
				thePlasmids[i].centerGroup.addChild(thePlasmids[i].lightBox);
			} else {
				thePlasmids[i].centerGroup.addChild(thePlasmids[i].lightSphere);
			}
			thePlasmids[i].g3d.addChild(thePlasmids[i].centerGroup);
		}
	}
	
	public void setGraphicsCapabilities() {
		super.setGraphicsCapabilities();
		centerGroup.setCapability(BranchGroup.ALLOW_DETACH);
		centerGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
	}
	
	public void makeGraphics () {
		// make material
		m = new Material (ambientC,emissiveC,diffuseC,specularC,shiny);
		// set capabilities
		setGraphicsCapabilities();
		
		ColoringAttributes cA = new ColoringAttributes(Env.plasmidColor3f, ColoringAttributes.NICEST);
		a.setColoringAttributes(cA);
		TransparencyAttributes tA = new TransparencyAttributes ();
		tA.setTransparency(0.8f);
		tA.setTransparencyMode(tA.NICEST);
		a.setTransparencyAttributes(tA);
		a.setMaterial(m);
		PolygonAttributes plasAtts = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_FRONT,0.0f);
		plasAtts.setBackFaceNormalFlip(true);
		a.setPolygonAttributes(plasAtts);
		mySphere = new Sphere((float)getRadius(),Sphere.GENERATE_NORMALS, Env.plasmidTessalation, a);
		mySphere.setCapability(Sphere.ALLOW_BOUNDS_READ);
		
		Material iMat = new Material(ambientIC,emissiveIC,diffuseIC,specularIC,shiny);
		Appearance iA = new Appearance();
		PolygonAttributes iAAtts = new PolygonAttributes(PolygonAttributes.POLYGON_FILL,PolygonAttributes.CULL_BACK,0.0f);
		plasAtts.setBackFaceNormalFlip(true);
		iA.setPolygonAttributes(iAAtts);
		iA.setMaterial(iMat);    
		//iA.setTransparencyAttributes(tA);
		float lightRad = (float)(getRadius()/3);
		lightSphere = new Sphere((float)(getRadius()/3),Sphere.GENERATE_NORMALS, 12, iA);
		lightBox = new Box(lightRad,lightRad,lightRad,Box.GENERATE_TEXTURE_COORDS,iA);
		if (Env.plasmidFaces) {
			centerGroup.addChild(lightBox);
		} else {
			centerGroup.addChild(lightSphere);
		}
		
		// put faces on the box
		if (!cubeAppsMade) { makeCubeApps(); }
		Shape3D face;
		for (int i=0;i<6;i++) {
			face = lightBox.getShape(i);
			face.setAppearance(cubeApps[i]);
		}
		
		// ParR-ParC spheres
		ColoringAttributes parCA = new ColoringAttributes(Env.parRparCColor3f, ColoringAttributes.NICEST);
		Appearance parRparCApp = new Appearance();
		parRparCApp.setColoringAttributes(cA);
		for (int i=0;i<parRparCSpheres.length;i++) {
			//parRparCSpheres[i] = new Sphere((float)Env.parRparCRadius,Sphere.GENERATE_NORMALS,Env.plasmidTessalation,parRparCApp);
		}
		
		// fiduciary marks
		double markSphereSize = getRadius()/12.0;
		Appearance yApp = new Appearance();
		ColoringAttributes yCA = new ColoringAttributes(new Color3f(Color.BLACK), ColoringAttributes.NICEST);
		Material yM = new Material (new Color3f(Color.BLACK),emissiveC,new Color3f(Color.BLACK),specularC,shiny);
		yApp.setColoringAttributes(yCA);
		yApp.setMaterial(yM);
		Sphere yMark = new Sphere((float)(markSphereSize),Sphere.GENERATE_NORMALS, Env.plasmidTessalation, yApp);
		Transform3D yMarkT3D = new Transform3D();
		yMarkT3D.setScale(new Vector3d(1.0,0.1,1.0));
		yMarkT3D.setTranslation(new Vector3d(0.0,getRadius(),0.0));
		TransformGroup yMarkTG = new TransformGroup();
		yMarkTG.addChild(yMark);
		yMarkTG.setTransform(yMarkT3D);
		
		Sphere xMark = new Sphere((float)(markSphereSize),Sphere.GENERATE_NORMALS, Env.plasmidTessalation, yApp);
		Transform3D xMarkT3D = new Transform3D();
		xMarkT3D.setScale(new Vector3d(0.1,1.0,1.0));
		xMarkT3D.setTranslation(new Vector3d(getRadius(),0.0,0.0));
		TransformGroup xMarkTG = new TransformGroup();
		xMarkTG.addChild(xMark);
		xMarkTG.setTransform(xMarkT3D);
		
		Appearance textAppear = new Appearance();
	    ColoringAttributes textColor = new ColoringAttributes();
	    textColor.setColor(1.0f, 0.0f, 0.0f);
	    textAppear.setColoringAttributes(textColor);
	    textAppear.setMaterial(new Material());
	    // Create a simple shape leaf node, add it to the scene graph.
	    Font3D font3D = new Font3D(new Font("Helvetica", Font.PLAIN, 1),new FontExtrusion());
	    Text3D yTextGeom = new Text3D(font3D, new String("Y"));
	    yTextGeom.setAlignment(Text3D.ALIGN_CENTER);
	    Shape3D yTextShape = new Shape3D();
	    yTextShape.setGeometry(yTextGeom);
	    yTextShape.setAppearance(textAppear);
	    Transform3D yTextT3D = new Transform3D();
	    TransformGroup yTextTG = new TransformGroup();
	    yTextT3D.rotX(Math.PI/2.0);
	    yTextT3D.setScale(0.015);
	    yTextT3D.setTranslation(new Vector3d(0,.010,-0.005));
	    yTextTG.setTransform(yTextT3D);
	    yTextTG.addChild(yTextShape);
	    yMarkTG.addChild(yTextTG);
		
	    Text3D xTextGeom = new Text3D(font3D, new String("X"));
	    xTextGeom.setAlignment(Text3D.ALIGN_CENTER);
	    Shape3D xTextShape = new Shape3D();
	    xTextShape.setGeometry(xTextGeom);
	    xTextShape.setAppearance(textAppear);
	    Transform3D xTextT3D = new Transform3D();
	    TransformGroup xTextTG = new TransformGroup();
	    xTextT3D.rotY(Math.PI/2.0);
	    xTextT3D.setScale(0.015);
	    xTextT3D.setTranslation(new Vector3d(0.007,-.005,0));

	    xTextTG.setTransform(xTextT3D);
	    xTextTG.addChild(xTextShape);
	    xMarkTG.addChild(xTextTG);
	    
	    // point light
	    for (int i=0;i<pLightCt;i++){
		    pLights[i] = new PointLight(Env.plasmidLightColor3f,new Point3f(0f,0f,0f),new Point3f(1f,1f,1f));
		    pLights[i].setCapability(PointLight.ALLOW_STATE_WRITE);
		    pLights[i].setCapability(PointLight.ALLOW_POSITION_WRITE);
		    pLights[i].setCapability(Light.ALLOW_SCOPE_WRITE);
		    pLights[i].removeAllScopes();
		   // pLights[i].addScope(Bug.bugG);
		    BoundingSphere plasLightBounds = new BoundingSphere(new Point3d( 0.0, 0.0, 0.0 ),3.0 );
		    pLights[i].setInfluencingBounds(plasLightBounds);
		    pLights[i].setAttenuation(0f,20f,0f);
	    }
	    float radMult = 1.1f;
	    float radiusf = (float)getRadius();
	    pLights[0].setPosition(radMult*radiusf,0,0);
	    pLights[1].setPosition(-radMult*radiusf,0,0);
	   // pLights[3].setPosition(0,-radMult*radiusf,0);
	   // pLights[4].setPosition(0,0,radMult*radiusf);
	   // pLights[5].setPosition(0,0,-radMult*radiusf);
		
		g3d.addChild(mySphere);
		g3d.addChild(centerGroup);
		//g3d.addChild(xMarkTG);
		//g3d.addChild(yMarkTG);
		for (int i=0;i<pLightCt;i++) { g3d.addChild(pLights[i]); }
		G.addChild(g3d);
		
		//pLight.removeAllScopes();
		
		graphicsMade = true;
	}
	public void updateGraphics () {
		/*if (mySphere.getRadius()-radius < radDrawTolerance) {
			g3d.removeChild(mySphere);
			mySphere = new Sphere((float)radius, Sphere.GENERATE_NORMALS, 80, aSph);
			g3d.addChild(mySphere);
		 }*/
		//t3d.setTranslation(getGraphicsLoc(coord));
		coord.copyToVector3d(coordVec3d);
		t3d.setTranslation(coordVec3d);
		t3d.setRotation(mxToX);
		g3d.setTransform(t3d);
	 }
	
	public static void makeInitialPlasmids() {
		for (int i=0;i<Env.initialPlasmids.getValue();i++) {
			makeRandomPlasmid();
		}
	}
	
	public static void makeRandomPlasmid () {
		Pt3D randomLoc = Thing.theBug.rdmPtInsideBug(Env.plasmidRadius.getValue());
		new Plasmid (randomLoc);
	}
	
	public static void makeTestPlasmidPair () {
		Pt3D loc1 = new Pt3D (Env.tstPlasmidOffset,0,0);
		Pt3D loc2 = new Pt3D (-Env.tstPlasmidOffset,0,0);
		new Plasmid(loc1);
		new Plasmid(loc2);
	}
	
	public static void addPlasmid (Plasmid newPlasmid) {
		thePlasmids[plasmidCt] = newPlasmid;
		plasmidCt ++;
	}
	
	public static void removeAll () {
		for (int i=0;i<plasmidCt;i++) {
			thePlasmids[i].G.detach();
		}
		for (int i=0;i<plasmidCt;i++) {
			thePlasmids[i].removeMe = true;
			thePlasmids[i] = null;
		}
		plasmidCt = 0;
	}
}
